""" aesthetics.py """


CCG_COLORS = {'pairs': 'blue', 'self': 'green'}

SYN_STRENGTH_COLORS = {'original': 'blue', 'jittered': 'green', 'ci': 'red', 'rectangle': 'lightgrey'}

TIME_UNIT_TIMELAG_LABELS = {'ms': 'Time lag (ms)', 's': 'Time lag (s)'}
